/*
    DefeoB - is a chess variants playing program
    Copyright (C) 2010 SKAcz.
	  Originally based on Sjeng85 Copyright (C) 2000-2001 Gian-Carlo Pascutto
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                          
    File: DefeoB.h
    Purpose: global definitions                  

*/

#ifndef DefeoB_H
#define DefeoB_H

#include "config.h"
#include <ctype.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#ifdef HAVE_SYS_TIMEB_H
#include <sys/timeb.h>
#endif

#define NDEBUG 
#include <assert.h>

#define DIE (*(int *)(NULL) = 0)

/* GCP : my code uses WHITE=0 and BLACK=1 so reverse this */

#define WHITE 0
#define BLACK 1

#define ToMove (white_to_move ? 0 : 1)
#define NotToMove (white_to_move ? 1 : 0)

#define Hash(x,y) (hash ^= zobrist[(x)][(y)])

#define Bughouse 1


#define Opening      0
#define Middlegame   1
#define Endgame      2

#define maxdepth 40
#define mindepth 2

/* define names for piece constants: */
#define frame   0
#define wpawn   1
#define bpawn   2
#define wknight 3
#define bknight 4
#define wking   5
#define bking   6
#define wrook   7
#define brook   8
#define wqueen  9
#define bqueen  10
#define wbishop 11
#define bbishop 12
#define npiece  13
// who&1 is like who%2 true if white pieces
/* result flags: */
#define no_result      0
#define stalemate      1
#define white_is_mated 2
#define black_is_mated 3
#define draw_by_fifty  4
#define draw_by_rep    5


#define sqA1 26  // lowest number of real chessboard square
#define sqH8 117 // most high number of real chessboard square
#define	sqA2	38
#define	sqA3	50
#define	sqA4	62
#define	sqA5	74
#define	sqA6	86
#define	sqA7	98
#define	sqA8	110
#define	sqB1	27
#define	sqB2	39
#define	sqB3	51
#define	sqB4	63
#define	sqB5	75
#define	sqB6	87
#define	sqB7	99
#define	sqB8	111
#define	sqC1	28
#define	sqC2	40
#define	sqC3	52
#define	sqC4	64
#define	sqC5	76
#define	sqC6	88
#define	sqC7	100
#define	sqC8	112
#define	sqD1	29
#define	sqD2	41
#define	sqD3	53
#define	sqD4	65
#define	sqD5	77
#define	sqD6	89
#define	sqD7	101
#define	sqD8	113
#define	sqE1	30
#define	sqE2	42
#define	sqE3	54
#define	sqE4	66
#define	sqE5	78
#define	sqE6	90
#define	sqE7	102
#define	sqE8	114
#define	sqF1	31
#define	sqF2	43
#define	sqF3	55
#define	sqF4	67
#define	sqF5	79
#define	sqF6	91
#define	sqF7	103
#define	sqF8	115
#define	sqG1	32
#define	sqG2	44
#define	sqG3	56
#define	sqG4	68
#define	sqG5	80
#define	sqG6	92
#define	sqG7	104
#define	sqG8	116
#define	sqH1	33
#define	sqH2	45
#define	sqH3	57
#define	sqH4	69
#define	sqH5	81
#define	sqH6	93
#define	sqH7	105



/* arrays maybe ? */ 
#ifdef FASTCALC
#define rank(square) ((((square)-26)/12)+1)
#define file(square) ((((square)-26)%12)+1)
#else
#define rank(square) (rank[(square)])
#define file(square) (file[(square)])
#endif

typedef enum {FALSE, TRUE} bool;

/* castle flags: */
#define no_castle  0
#define wck        1
#define wcq        2
#define bck        3
#define bcq        4

typedef struct 
{
  int from;
  int target;
  int captured;
  int promoted;	      
  int castled;
  int ep; 
} move_s;

typedef struct 
{
  int cap_num;
  int was_promoted;
  int epsq;
  int fifty;
} move_x;

#if defined(HAVE_SYS_TIMEB_H) && (defined(HAVE_FTIME) || defined(HAVE_GETTIMEOFDAY)) 
typedef struct timeb rtime_t;
#else
typedef time_t rtime_t;
#endif

#define STR_BUFF 256
#define MOVE_BUFF 300
#define INF 1000000
#define PV_BUFF 100

#define AddMaterial(x) Material += material[(x)]
#define RemoveMaterial(x) Material -= material[(x)]

#define UPPER 1
#define LOWER 2
#define EXACT 3
#define HMISS 4
#define DUMMY 0

#define LOSS 0
#define WIN 1
#define DRAW 2

#endif
