/*
    DefeoB - a bughouse chess playing program
    Copyright (C) 2010 SKAcz.
	  Originally based on Sjeng85 Copyright (C) 2000-2001 Gian-Carlo Pascutto

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    File: search.c                                        
    Purpose: contains functions related to the recursive search

*/

#include "DefeoB.h"
#include "extvars.h"
#include "protos.h"
#include "limits.h"

unsigned long FH, FHF;
unsigned long razor_drop, razor_material, drop_cuts;
char true_i_depth;
int bestmovenum;
int ugly_ep_hack;
char postpv[STR_BUFF];
#define KINGCAP 50000



void order_moves (move_s moves[], long int move_ordering[], int num_moves, int best) 
{ /* sort out move ordering scores in move_ordering, using implemented heuristics: */
  //                       frame,P  ,p  ,N  ,n  ,K    ,k    ,R  ,r  ,Q  ,q  ,B  ,b
  static int cap_values[14] = {0,100,100,210,210,50000,50000,250,250,450,450,230,230,0};
  static int cap_index[14] = {0,1,1,2,2,6,6,4,4,5,5,3,3,0};
  int nearWK, nearBK;
	    
  int promoted, captured, i, from, target, who;


  /* fill the move ordering array: */
  /* if searching the pv, give it the highest move ordering, and if not, rely on the other heuristics: */
  if (DebugRunST && DebugRunOutput<DebugRunOutputMax) {printf ("searching_pv:%d]",searching_pv);DebugRunOutput++;}

  
  if (searching_pv) 
  { searching_pv = FALSE;
    for (i = 0; i < num_moves; i++) 
    { from = moves[i].from; target = moves[i].target; promoted = moves[i].promoted; captured = moves[i].captured;
      nearWK=target-wking_loc; nearBK=target-bking_loc; who=board[from];
	    //Here ordering for bughouse drop pieces near king especially with check has to be given precedence
      if (from==0) //Drop pieces
	    { 
        move_ordering[i]=0;
	      //Knights attacks
		    if (promoted==wknight)
		    { 
			    if (nearBK==(sqE5-sqF7))
               move_ordering[i]=50000+10000+400;
			    else if (nearBK==(sqG5-sqF7))
               move_ordering[i]=50000+10000+399;
			    else if (nearBK==(sqC7-sqE8))
               move_ordering[i]=50000+10000+398;
			    else if (nearBK==(sqG7-sqE8))
               move_ordering[i]=50000+10000+397;
          else if (nearBK==(sqD8-sqF7))
               move_ordering[i]=50000+10000+396;
          else if (nearBK==(sqH8-sqF7))
               move_ordering[i]=50000+10000+395;
          else if (nearBK==(sqG8-sqH6))
               move_ordering[i]=50000+10000+394; 
          else if (nearBK==(sqH8-sqG6))
               move_ordering[i]=50000+10000+393;
              
		    }
        else if (promoted==bknight)
		    {
			    if (nearWK==(sqE4-sqF2))
               move_ordering[i]=50000+10000+400;
			    else if (nearWK==(sqG4-sqF2))
               move_ordering[i]=50000+10000+399;
			    else if (nearWK==(sqC2-sqE1))
               move_ordering[i]=50000+10000+398;
			    else if (nearWK==(sqG2-sqE1))
               move_ordering[i]=50000+10000+397;
			    else if (nearWK==(sqD1-sqF2))
               move_ordering[i]=50000+10000+396; 
			    else if (nearWK==(sqH1-sqF2))
               move_ordering[i]=50000+10000+395;                
			    else if (nearWK==(sqG1-sqH3))
               move_ordering[i]=50000+10000+394; 
			    else if (nearWK==(sqH1-sqG3))
               move_ordering[i]=50000+10000+393;                
		    }
		    else if (promoted==wqueen)
		    {
         // contact checks
			    if (nearBK==(-11))
               move_ordering[i]=50000+10000+400;
			    else if (nearBK==(-12))
               move_ordering[i]=50000+10000+399;
			    else if (nearBK==(-13))
               move_ordering[i]=50000+10000+398;
			    else if (nearBK==(+1))
               move_ordering[i]=50000+10000+397;
			    else if (nearBK==(-1))
               move_ordering[i]=50000+10000+396;
                else if (nearBK==(+13))
               move_ordering[i]=50000+10000+395;
			    else if (nearBK==(+12))
               move_ordering[i]=50000+10000+394;
			    else if (nearBK==(+11))
               move_ordering[i]=50000+10000+393;          
          // checks like f8-h8
			    else if (board[bking_loc-1]==npiece && board[bking_loc-13]==npiece && nearBK==(-2))
               move_ordering[i]=50000+10000+200;               
			    else if (board[bking_loc+1]==npiece && board[bking_loc+13]==npiece && nearBK==(+2))
               move_ordering[i]=50000+10000+199;                 
		    }
		    else if (promoted==bqueen)
		    {
         // contact checks
			    if (nearWK==(11))
               move_ordering[i]=50000+10000+400;
			    else if (nearWK==(13))
               move_ordering[i]=50000+10000+399;
			    else if (nearWK==(12))
               move_ordering[i]=50000+10000+398;
			    else if (nearWK==(1))
               move_ordering[i]=50000+10000+397;
			    else if (nearWK==(-1))
               move_ordering[i]=50000+10000+396;
                else if (nearWK==(-11))
               move_ordering[i]=50000+10000+395;
			    else if (nearWK==(-13))
               move_ordering[i]=50000+10000+394;
			    else if (nearWK==(-12))
               move_ordering[i]=50000+10000+393;                
          // checks like f1-h1
			    else if (board[wking_loc-1]==npiece && board[wking_loc-13]==npiece && nearWK==(-2))
               move_ordering[i]=50000+10000+200;               
			    else if (board[wking_loc+1]==npiece && board[wking_loc+13]==npiece && nearWK==(+2))
               move_ordering[i]=50000+10000+199;                       
		    }
		    else if (promoted==wpawn)
		    {
			    if (nearBK==(-11))
               move_ordering[i]=50000+10000+402;
			    else if (nearBK==(-13))
               move_ordering[i]=50000+10000+401;
		    }
		    else if (promoted==wbishop)
		    {
			    if (nearBK==(-11))
               move_ordering[i]=50000+10000+402;
			    else if (nearBK==(-13))
               move_ordering[i]=50000+10000+401;
			    if (nearBK==(11))
               move_ordering[i]=50000+10000+300;
			    else if (nearBK==(13))
               move_ordering[i]=50000+10000+299;                    
		    }        
		    else if (promoted==bpawn)
		    {
			    if (nearWK==(11))
               move_ordering[i]=50000+10000+402;
			    else if (nearWK==(13))
               move_ordering[i]=50000+10000+401;
		    }        
		    else if (promoted==bbishop)
		    {
			    if (nearWK==(11))
               move_ordering[i]=50000+10000+402;
			    else if (nearWK==(13))
               move_ordering[i]=50000+10000+401;
			    if (nearWK==(11))
               move_ordering[i]=50000+10000+300;
			    else if (nearWK==(13))
               move_ordering[i]=50000+10000+299;                      
		    }
		    else if (promoted==wrook)
		    {
			    if (nearBK==(-12))
               move_ordering[i]=50000+10000+396;
			    else if (nearBK==(-1))
               move_ordering[i]=50000+10000+395;
			    else if (nearBK==(+1))
               move_ordering[i]=50000+10000+394;
		    }
		    else if (promoted==brook)
		    {
			    if (nearWK==(12))
               move_ordering[i]=50000+10000+396;
			    else if (nearWK==(-1))
               move_ordering[i]=50000+10000+395;
			    else if (nearWK==(+1))
               move_ordering[i]=50000+10000+394;
		    }
      }
	    else //no drop
      { 
        move_ordering[i] = 0;
        if (captured != npiece) //capturing 
	      { //give captures precedence in move ordering, and order captures by material gain 
          move_ordering[i] = cap_values[captured]-cap_index[board[from]]+50000;
      
          //in bughouse capturing near king is important
		      if (who&1 && (nearBK==(-13) || nearBK==(-12) || nearBK==(-11) || nearBK==(-1) || nearBK==(1)) )
              move_ordering[i]+= 5000;
		      else if (!(who&1) && (nearBK==(13) || nearBK==(12) || nearBK==(11) || nearBK==(1) || nearBK==(-1)) )
             move_ordering[i]+= 5000;
             
          //capturing with check is very good      
                if (who==wpawn)
                {
                  if (nearBK==(-11))
                       move_ordering[i]+=10000+402;
                  else if (nearBK==(-13))
                       move_ordering[i]+=10000+401;
                }
                else if (who==wbishop)
                {
                  if (nearBK==(-11))
                       move_ordering[i]+=10000+402;
                  else if (nearBK==(-13))
                       move_ordering[i]+=10000+401;
                  else if (nearBK==(13))
                       move_ordering[i]+=10000+400;                       
                  else if (nearBK==(11))
                       move_ordering[i]+=10000+399;                       
                }
                else if (who==bpawn)
                {
                  if (nearWK==(11))
                       move_ordering[i]+=10000+402;
                  else if (nearWK==(13))
                       move_ordering[i]+=10000+401;
                }
                else if (who==bbishop)
                {
                  if (nearWK==(11))
                       move_ordering[i]+=10000+402;
                  else if (nearWK==(13))
                       move_ordering[i]+=10000+401;
                  else if (nearWK==(-13))
                       move_ordering[i]+=10000+400;
                  else if (nearWK==(-11))
                       move_ordering[i]+=10000+399;                   
                }                
                else if (who==wknight)
                {
                  if (nearBK==(sqE5-sqF7))
                       move_ordering[i]+=10000+400;
                  else if (nearBK==(sqG5-sqF7))
                       move_ordering[i]+=10000+399;
                  else if (nearBK==(sqC7-sqE8))
                       move_ordering[i]+=10000+398;
                  else if (nearBK==(sqG7-sqE8))
                       move_ordering[i]+=10000+397;
                  else if (nearBK==(sqD8-sqF7))
                       move_ordering[i]+=10000+396;
                  else if (nearBK==(sqH8-sqF7))
                       move_ordering[i]+=10000+395;
                  else if (nearBK==(sqG8-sqH6))
                       move_ordering[i]+=10000+394; 
                  else if (nearBK==(sqH8-sqG6))
                       move_ordering[i]+=10000+393;
                }
                else if (who==bknight)
                {
        			    if (nearWK==(sqE4-sqF2))
                       move_ordering[i]+=10000+400;
        			    else if (nearWK==(sqG4-sqF2))
                       move_ordering[i]+=10000+399;
        			    else if (nearWK==(sqC2-sqE1))
                       move_ordering[i]+=10000+398;
        			    else if (nearWK==(sqG2-sqE1))
                       move_ordering[i]+=10000+397;
        			    else if (nearWK==(sqD1-sqF2))
                       move_ordering[i]+=10000+396; 
        			    else if (nearWK==(sqH1-sqF2))
                       move_ordering[i]+=10000+395;                
        			    else if (nearWK==(sqG1-sqH3))
                       move_ordering[i]+=10000+394; 
        			    else if (nearWK==(sqH1-sqG3))
                       move_ordering[i]+=10000+393;
                }
                
        
	      }
         
         //move with check or discovered check can be very good
         
         
         //move with discovered check 
         
           //NB,RB
           if (who&1) //white move
           {
             if (who==wknight || who==wrook)
             { //for example Ka1,nb2,bc3  move nb2.... is discovered check                
               if (nearBK==(sqH1-sqG2) && (board[bking_loc+(sqH1-sqF3)]==bbishop || board[bking_loc+(sqH1-sqF3)]==bqueen) )
                   move_ordering[i]+=50000+20000+100;
               else if (nearBK==(sqA1-sqB2) && (board[bking_loc+(sqA1-sqC3)]==bbishop || board[bking_loc+(sqA1-sqC3)]==bqueen) )
                  move_ordering[i]+=50000+20000+99;
               else if (nearBK==(sqB2-sqA1) && (board[bking_loc+(sqC3-sqA1)]==bbishop || board[bking_loc+(sqC3-sqA1)]==bqueen) )
                  move_ordering[i]+=50000+20000+98;             
               else if (nearBK==(sqG2-sqH1) && (board[bking_loc+(sqF3-sqH1)]==bbishop || board[bking_loc+(sqF3-sqH1)]==bqueen) )
                  move_ordering[i]+=50000+20000+97; 
             } 
             else if (who==wpawn || who==wbishop)
			 {
			  if (nearBK==(sqF7-sqE8))
			    move_ordering[i]+=50000+10000+99; //brX XXX
			  else if (nearBK==(sqF7-sqG8))
			    move_ordering[i]+=50000+10000+498;
			 }
           }
           else // black move
           {
             if (who==bknight || who==brook)
             { //for example Ka1,nb2,bc3  move nb2.... is discovered check           
               if (nearWK==(sqB2-sqA1) && (board[wking_loc+(sqC3-sqA1)]==bbishop || board[wking_loc+(sqC3-sqA1)]==bqueen) )
                  move_ordering[i]+=50000+20000+100;
               else if (nearWK==(sqG2-sqH1) && (board[wking_loc+(sqF3-sqH1)]==bbishop || board[wking_loc+(sqF3-sqH1)]==bqueen) )
                  move_ordering[i]+=50000+20000+99;
               else if (nearWK==(sqA1-sqB2) && (board[wking_loc+(sqA1-sqC3)]==bbishop || board[wking_loc+(sqA1-sqC3)]==bqueen) )
                  move_ordering[i]+=50000+20000+98;
               else if (nearWK==(sqH1-sqG2) && (board[wking_loc+(sqH1-sqF3)]==bbishop || board[wking_loc+(sqH1-sqF3)]==bqueen) )
                  move_ordering[i]+=50000+20000+97;                  
             }
             else if (who==bpawn || who==bbishop)
			 {
			  if (nearBK==(sqF2-sqE1))
			    move_ordering[i]+=50000+10000+99;
			  else if (nearBK==(sqF2-sqG1))
			    move_ordering[i]+=50000+10000+98;
			 }
           }
         
       
        
	       //move without check or capturing 

      }
      
      

     
      
      
      /* make the pv have highest move ordering: */
      if (from == pv[1][ply].from && target == pv[1][ply].target
	        && promoted == pv[1][ply].promoted) 
      { searching_pv = TRUE; move_ordering[i] += INF;
      } 
      else if ((best != -1) && (best != -2) && (i == best))
	    { move_ordering[i] += INF;
	    }
      else if (best == -2)
      { /* we have an iterative deepening move */
	      if (from == pv[ply+1][ply+1].from && target == pv[ply+1][ply+1].target 
	          && promoted == pv[ply+1][ply+1].promoted)
	      { move_ordering[i] += INF;
	      }
	    }  
      /* heuristics other than pv (no need to use them on the pv move - it is already ordered highest) */
      else 
      { /* add the history heuristic bonus: */
	      move_ordering[i] += (history_h[from][target]>>i_depth);
      	/* add the killer move heuristic bonuses: */
	      if (from == killer1[ply].from && target == killer1[ply].target
	          && promoted == killer1[ply].promoted)
	        move_ordering[i] += 10000;
	      else if (from == killer2[ply].from && target == killer2[ply].target
	               && promoted == killer2[ply].promoted)
	        move_ordering[i] += 5000;
	      else if (from == killer3[ply].from && target == killer3[ply].target
	               && promoted == killer3[ply].promoted)
	        move_ordering[i] += 2500;
      }

    } //for moves end
  } //searching pv end
  /* if not searching the pv: */
  else 
  { // no searching pv
    for (i = 0; i < num_moves; i++) 
    { from = moves[i].from; target = moves[i].target;
      promoted = moves[i].promoted; captured = moves[i].captured;  
      /* give captures precedence in move ordering, and order captures by material gain */
      if ((best != -1) && (i == best))
	    { move_ordering[i] += INF;
	    }
      else if (best == -2)
	    { /* we have an iterative deepening move */
	      if (from == pv[ply+1][ply+1].from && target == pv[ply+1][ply+1].target 
	          && promoted == pv[ply+1][ply+1].promoted)
	      { move_ordering[i] += INF;
	      }
	    }
      else if (captured != npiece)
	    { move_ordering[i] = cap_values[captured]-cap_index[board[from]]+50000;	    
	    }      
      else
	    { move_ordering[i] = 0;
      }      
      // heuristics other than pv 
      // add the history heuristic bonus:
       move_ordering[i] += (history_h[from][target]>>i_depth);
      // add the killer move heuristic bonuses: 
      if (from == killer1[ply].from && target == killer1[ply].target
	        && promoted == killer1[ply].promoted)
	    {move_ordering[i] += 10000;
      }
      else if (from == killer2[ply].from && target == killer2[ply].target
	            && promoted == killer2[ply].promoted)
	    { move_ordering[i] += 5000;
      }
      else if (from == killer3[ply].from && target == killer3[ply].target
	             && promoted == killer3[ply].promoted)
	    {move_ordering[i] += 2500;
      }
    } //for moves end
  } //no searching pv end

  
  if (DebugRunST && DebugRunOutput<DebugRunOutputMax)
  {
    

	long int move_ordering_tmp[1000];
    int j,ibest=0;
	for (i = 0; i < num_moves; i++) move_ordering_tmp[i]=move_ordering[i];
	

    printf(" Ordering[moves:%d",num_moves);
	for (i = 0; i < num_moves; i++)
	{
	 ibest=i;
     for (j = 0; j < num_moves; j++)
      if (move_ordering_tmp[j]>move_ordering_tmp[ibest]) {ibest=j;};
	  
	  if (DebugRunST && DebugRunOutput<DebugRunOutputMax)
		 printf ("[%s %d]",comp_to_coord2 (moves[ibest]),move_ordering[ibest]);DebugRunOutput++;
      move_ordering_tmp[ibest]=-999999999;
	}
    printf ("]");
  }
  
}





void perft (int depth) 
{
  move_s moves[MOVE_BUFF];
  int num_moves, i, ep_temp;

  if (DebugRun) printf (" perft(depth=%d)\n{\n",depth);

  ep_temp = ep_square; num_moves = 0;
  /* return if we are at the maximum depth: */
  if (!depth) 
  {
	  if (DebugRun) printf ("perft_ln257end}\n");
	  return;
  }
  /* generate the move list: */
  gen (&moves[0]); num_moves = numb_moves;
  /* loop through the moves at the current depth: */
  for (i = 0; i < num_moves; i++) 
  { make (&moves[0], i);
    if (DebugRun) printf ("\n perft_move[%s]\n{",comp_to_coord2 (moves[i]));
    if (check_legal (&moves[0], i)) /* check to see if our move is legal: */
	  { raw_nodes++;
      /* go deeper into the tree recursively, increasing the indent to create the "tree" effect: */
      perft (depth-1);
    }   
    unmake (&moves[0], i); /* unmake the move to go onto the next: */
  }
  ep_square = ep_temp;
  if (DebugRun) printf ("perft_end}\n");
}

long int qsearch (int alpha, int beta, int depth) 
{ /* perform a quiscense search on the current node using alpha-beta with negamax search */
  move_s moves[MOVE_BUFF];
  int num_moves, i, j;
  long int score = -INF, standpat, move_ordering[MOVE_BUFF];
  bool legal_move, no_moves = TRUE;
  int sbest, best_score, best, delta, bound, originalalpha;

  if (DebugRun) printf (" qsearch(alpha=%d,beta=%d,depth=%d)\n{\n",alpha,beta,depth);

  pv_length[ply] = ply;
  /* return our score if we're at a leaf node: */
  if (depth <= 0) 
  { /* remove leafcounting effect */
    qnodes--; score = eval (); 
	  if (DebugRun) printf (" qsearch_ln290 return score(%d)}\n",score);	
	return score;
  }
  /* before we do anything, see if we're out of time: */
  if (!(nodes & 4095)) 
  {
    if (((interrupt() && !go_fast) || (rdifftime (rtime (), start_time) >= time_for_move)) && (i_depth > 1)) 
	  { time_exit = TRUE; 
	    if (DebugRun) printf (" qsearch_ln298 return 0}\n");		
	    return 0;
	  }
  }
  originalalpha = alpha;
  switch (QProbeTT(&bound, alpha, beta, &best))
  { case EXACT:
      if (DebugRun) printf (" qsearch_ln305 exact return(%d)}\n",bound);	
      return bound;
    break;
    case UPPER:
      if (bound <= alpha) 
	  {  if (DebugRun) printf (" qsearch_ln310 upper return(%d)}\n",bound);	
	     return bound;
	  }
      if (bound < beta) beta = bound;
    break;
    case LOWER:
      if (bound >= beta)
	  {	  if (DebugRun) printf (" qsearch_ln317 lower return(%d)}\n",bound);	
		  return bound;
	  }
      if (bound > alpha) alpha = bound;
      break;
    case DUMMY:
    break;
    case HMISS:
      best = -1;;
    break;
  };
  standpat = eval();
  if (standpat >= beta) 
  {   if (DebugRun) printf (" qsearch_ln290 standpat return(%d)}\n",standpat);	
	  return standpat;        /* standpat */
  }
  else if (standpat > alpha) 
  { alpha = standpat;
  }
  num_moves = 0; sbest = -1; best_score = -INF; delta =alpha-200-standpat;
  /* generate and order moves: */
  gen (&moves[0]); num_moves = numb_moves;
  if (kingcap) 
  {   if (DebugRun) printf (" qsearch_ln290 kingcap return(%d)}\n",kingcap); 
	  return KINGCAP;
  }
  if (DebugRunST && DebugRunOutput<DebugRunOutputMax) printf (" qsearchL548.order_moves[%d]",num_moves);
  order_moves (&moves[0], &move_ordering[0], num_moves, best);
  /* loop through the moves at the current node: */
  while (remove_one (&i, &move_ordering[0], num_moves)) 
  {   if (DebugRun) printf ("\n qsearch_ln327_move[%s]",comp_to_coord2 (moves[i]));
	  make (&moves[0], i); legal_move = FALSE;
      /* go deeper if it's a legal move: */
      /* check whether it is a futile capture : captured piece wont bring us back near alpha */
      /* warning : promotions should be accounted for */
      /* because the piece will go to our holdings, its value is essentially double */
      DeltaTries++;
    if ((abs(material[moves[i].captured] * 2) >= delta) || (moves[i].promoted))
	  { nodes++; qnodes++; score = -qsearch (-beta, -alpha, depth-1);
	    if (score != -KINGCAP) 
		  { nodes++; qnodes++; legal_move = TRUE; no_moves = FALSE;
		  };
	  }
    else DeltaCuts++;
    unmake (&moves[0], i);
    if(score > best_score && legal_move) best_score = score;
    /* check our current score vs. alpha: */
    if (score > alpha && legal_move) 
	  {  /* don't update the history heuristic scores here, since depth is messed  up when qsearch is called */
	     best = i;
	     /* try for an early cutoff: */
	     if (score >= beta) 
	     { QStoreTT(score, originalalpha, beta, i); 
		   if (DebugRun) printf (" qsearch_ln369 score>=beta return(%d)}\n",score);
		   return score;
	     }
	     alpha = score;
	     /* update the pv: */
	     pv[ply][ply] = moves[i];;
	     for (j = ply+1; j < pv_length[ply+1]; j++) pv[ply][j] = pv[ply+1][j];
	     pv_length[ply] = pv_length[ply+1];
	  }  
  }
  /* we don't check for mate / stalemate here, because without generating all
         of the moves leading up to it, we don't know if the position could have been avoided by one side or not */
  if (no_moves)
  {
      if (DebugRun) printf (" qsearch_ln383 standpat return(%d)}\n",standpat);
	  return standpat;
  }
  else
  {
    QStoreTT(alpha, originalalpha, beta, best); 
	 if (DebugRun) printf (" qsearch_ln389 alfa return(%d)}\n",alpha);
	return alpha;
  }  
}


bool remove_one (int *marker, long int move_ordering[], int num_moves) 
{ /* a function to give pick the top move order, one at a time on each call.
         Will return TRUE while there are still moves left, FALSE after all moves  have been used */
  int i, best = -1;
  *marker = -1;


  //SKAcz : count only moves with evaluation>0 or first 10. brX
  //if (num_moves>10) best=1; -not as simple has to be sorted first or cannot have -value

  for (i = 0; i < num_moves; i++)  
  { if (move_ordering[i] > best) 
	  { *marker = i; best = move_ordering[i];
    }
  }
  if (*marker > -1) 
  { 
	 //if (DebugRunST && DebugRunOutput<DebugRunOutputMax) 
	 // {printf ("[mo %d]",move_ordering[*marker]);DebugRunOutput++;};
	  move_ordering[*marker] = -1; return TRUE;
  }
  else 
    return FALSE;
}

long int search (int alpha, int beta, int depth, bool is_null) 
{ /* search the current node using alpha-beta with negamax search */
  move_s moves[MOVE_BUFF];
  int num_moves, i, j;
  long int score = -INF, move_ordering[MOVE_BUFF];
  bool no_moves, legal_move;
  int bound, threat, donull, best, sbest, best_score, old_ep;
  bool incheck, first;
  int extend, fscore, fmax, selective;
  move_s kswap;
  int ksswap;
  int originalalpha;
  int afterincheck;
  int legalmoves;
  int dropcut;
  static int rc_index[14] = {0,1,1,2,2,5,5,3,3,4,4,2,2,0};
  
  if (DebugRun) printf (" search (alpha=%d,beta=%d,depth=%d,is_null=%d)\n{\n",alpha,beta,depth,is_null);

  /* before we do anything, see if we're out of time: */
  if (!(nodes & 4095)) 
  { if (((interrupt() && !go_fast) || (rdifftime (rtime (), start_time) >= time_for_move)) && (i_depth > 1)) 
	{ time_exit = TRUE; 
         if (DebugRun) printf (" search_ln436 timeexit return(0)}\n");
        return 0;
    }
  }
  originalalpha = alpha; fmax = -INF; threat = 0; extend = 0; pv_length[ply] = ply;
  if (is_draw ()) 
  {	  if (DebugRun) printf (" search_ln442 is_draw return(0)}\n");
	  return 0;
  }
  /* can eliminate this */
   incheck = in_check(); checks[ply] = incheck;
  /* perform check extensions if we haven't gone past maxdepth: */
  if (ply < maxdepth+1 && incheck && (ply < (i_depth*2))) 
  {  depth++; ext_check++; extend++;
  } 
  /* try to find a stable position before passing the position to eval (): */
  if (depth <= 0) 
  { loudnesses = TRUE; score = qsearch (alpha, beta, i_depth*3); loudnesses = FALSE;
    if (DebugRun) printf (" search_ln454 score return(%d)}\n",score);
    return score; 
  }
  num_moves = 0; no_moves = TRUE;
  switch (ProbeTT(&bound, alpha, beta, &best, &threat, &donull, depth))
  { case EXACT:
      if (DebugRun) printf (" search_ln460 exact return(%d)}\n",bound);
      return bound;
    break;
    case UPPER:
      if (bound <= alpha) 
	  {   if (DebugRun) printf (" search_ln465 bound<=alpha return(%d)}\n",bound); 
		  return bound;
	  }
      if (bound < beta)  	beta = bound;
    break;
    case LOWER:
      if (bound >= beta)
	  {  if (DebugRun) printf (" search_ln472 bound>=beta return(%d)}\n",bound);
		 return bound;
	  }
      if (bound > alpha) alpha = bound;
    break;
    case DUMMY:
    break;
    case HMISS:
      best = -1; threat = FALSE;
    break;
  };
  if (best == 255) best = -1;
  sbest = -1; best_score = -INF; old_ep = ep_square;
  if ((is_null == FALSE) && !incheck && donull && (threat == FALSE))
  {   ep_square = 0; white_to_move ^= 1; ply++; fifty++; hash ^= 0xDEADBEEF;
	    /* use R=1 cos R=2 is too dangerous for our ply depths */
	    score = -search(-beta, -beta+1, ((depth > 3) ? depth-2-1 : depth-1-1), TRUE);
	    hash ^=0xDEADBEEF; fifty--; ply--; white_to_move ^= 1; ep_square = old_ep;
      if (time_exit) 
	  {
        if (DebugRun) printf (" search_ln492 timeexit return(0)}\n");		
		return 0;
	  }
      NTries++;
      if (score >= beta)
	    { NCuts++; StoreTT(score, alpha, beta, 255, 0, depth); 
	     if (DebugRun) printf (" search_ln498 score(%d)>=beta(%d) return(%d)}\n",score,beta,score);
	     return score;
	    }
      else if (score < -INF+100)
	    { threat = TRUE; TExt++; depth++; extend++;
	    }
  }
  else if (threat == TRUE)
  { TExt++; depth++; extend++;
  } 
#undef IDEEP
#ifdef IDEEP
  /* Internal Iterative deepening */
  if ((best == -1) && (depth > 2) && (searching_pv)) 
  {  score = search (alpha, beta, depth-2, TRUE); ep_square = old_ep;
     if (time_exit) 
	 {   if (DebugRun) printf (" search_ln514 timeexit return(0)}\n");
		 return 0;
	 }
     if (score <= alpha)
	 { score = search ( -INF, beta, depth-2, TRUE); ep_square = old_ep;	  
   	   if (time_exit)
	   {   if (DebugRun) printf (" search_ln520 timeexit return(0)}\n");
		   return 0;
	   }
	 }
     best = -2;
  }
#endif
  score = -INF; first = TRUE; selective = 0;
  if (cfg_futprune)
  { fscore = (white_to_move ? Material : -Material) + 900;
    if (!extend && depth == 3 && fscore <= alpha && !incheck) depth =2;
    fscore = (white_to_move ? Material : -Material) + 500;
    if (!extend && depth == 2 && fscore <= alpha)
    { selective = 1; best_score = fmax = fscore;
    }
    fscore = (white_to_move ? Material : -Material) + 200;
    if (!extend && depth == 1 && fscore <= alpha)
    { selective = 1; best_score = fmax = fscore;
    }
  }
  legalmoves = 0;
  /* generate and order moves: */
  gen (&moves[0]); num_moves = numb_moves;
  if (num_moves > 0)
  {
	if (DebugRunST && DebugRunOutput<DebugRunOutputMax) printf (" searchL758.order_moves");
    order_moves (&moves[0], &move_ordering[0], num_moves, best);
    /* loop through the moves at the current node: */
    while (remove_one (&i, &move_ordering[0], num_moves)) 
	  {  make (&moves[0], i); legal_move = FALSE;
	     if (DebugRun) printf ("\n search_ln495_move[%s]",comp_to_coord2 (moves[i]));
      hash_history[move_number+ply-1] = hash; path[ply-1] = moves[i];
      afterincheck = in_check(); old_ep = ep_square;
	    extend = 0; /* dont extend twice */
      /* go deeper if it's a legal move: */
      if (check_legal (&moves[0], i)) 
	    {	dropcut = 0;
	      /* Razoring of uninteresting drops */
         if ((moves[i].from == 0)
	           && (depth > 1)   /* more than pre-frontier nodes */
	           && !afterincheck   /* not a checking move */
	           && !incheck      /* not a check evasion */
	           && !searching_pv
             && cfg_razordrop
	          )
		    { razor_drop++; extend--;}
	      else
		    {
	       if ((moves[i].from == 0) && (depth == 1) && !incheck && cfg_cutdrop) 
		      {
	          if (white_to_move)
			      {
	            /* black dropped the piece */
	            /* check more white than black attackers */
	            dropcut =(calc_attackers(moves[i].target,1) -calc_attackers(moves[i].target, 0)) >0;
	            if (dropcut) drop_cuts++;
			      }
	          else
			      { dropcut =(calc_attackers(moves[i].target,0) -calc_attackers(moves[i].target,1)) >0;
	           if (dropcut) drop_cuts++;
			      }
		      }
		    }
	      if (!dropcut && (!selective || afterincheck
	          || (fmax + ((abs(material[moves[i].captured]) *2)) > alpha) 
	          || (moves[i].promoted))) 
		    {     
	       /* we only count the nodes we actually examine */    
	       nodes++; 
	        if (first == TRUE)
		      { ep_square = old_ep; 
		       score = -search (-beta, -alpha, depth+extend-1, FALSE);
		       ep_square = old_ep; FULL++;
		      }
	      else
		    { ep_square = old_ep;
		      score = -search (-alpha-1, -alpha, depth+extend-1, FALSE);
		      PVS++; ep_square = old_ep;	    
		      if (score > best_score && !time_exit && score != -KINGCAP)
			    {  best_score = score;
			        if ((score > alpha) && (score < beta))
			        { score = -search(-beta, -score, depth+extend-1, FALSE);
			         ep_square = old_ep;
			         PVSF++;  
			        }
			    }
		    }
	      legal_move = TRUE;
		    }
	      else
	        razor_material++;
		
	      legalmoves++;
	      no_moves = FALSE;
	    }
      if (score > best_score && legal_move) best_score = score;
      unmake (&moves[0], i);  
      /* return if we've run out of time: */
      if (time_exit) 
	  {   if (DebugRun) printf (" search_ln618 timeexit return(0)}\n"); 
		  return 0;
	  }
      /* check our current score vs. alpha: */
      if (score > alpha && legal_move) 
	    { /* try for an early cutoff: */
	      if (score >= beta) 
		    { /* update the history heuristic since we have a cutoff: */
	        history_h[moves[i].from][moves[i].target] += depth * depth;	  
	        if (moves[i].captured == npiece)
		      { /* we have a cutoff, so update our killers: */
	          /* first, check whether it matches one of the known killers */
	          if (moves[i].from == killer1[ply].from && moves[i].target ==
		            killer1[ply].target && moves[i].promoted == killer1[ply].promoted)
			      { killer_scores[ply]++;
			      }
	          else if (moves[i].from == killer2[ply].from && moves[i].target ==
		                 killer2[ply].target && moves[i].promoted == killer2[ply].promoted)
			      { killer_scores2[ply]++;
		          if (killer_scores2[ply] > killer_scores[ply])
			        { kswap = killer1[ply];
		            killer1[ply] = killer2[ply];
		            killer2[ply] = kswap;		
		            ksswap = killer_scores[ply];
		            killer_scores[ply] = killer_scores2[ply];
		            killer_scores2[ply] = ksswap;
			        }
			      }     
	          else if (moves[i].from == killer3[ply].from && moves[i].target ==
		                 killer3[ply].target && moves[i].promoted == killer3[ply].promoted)
			      { killer_scores3[ply]++;	  
		          if (killer_scores3[ply] > killer_scores2[ply])
			        { kswap = killer2[ply];
		           killer2[ply] = killer3[ply];
		           killer3[ply] = kswap;		
		           ksswap = killer_scores2[ply];
		           killer_scores2[ply] = killer_scores3[ply];
		           killer_scores3[ply] = ksswap;
			        }
			      }
	          /* if not, replace killer3 */
	          else
			      { killer_scores3[ply]=1; killer3[ply]=moves[i];
			      }
		      }
	        if (first == TRUE) FHF++; 
	        FH++;
	        StoreTT(score, originalalpha, beta, i, threat, depth); 
			if (DebugRun) printf (" search_ln666 score return(%d)}\n",score);
	        return score;
		    }
	      alpha=score; sbest = i;
	      /* update the pv: */
	      pv[ply][ply] = moves[i];
	      for (j = ply+1; j < pv_length[ply+1]; j++) pv[ply][j] = pv[ply+1][j];
	      pv_length[ply] = pv_length[ply+1];
	    } 
      if (legal_move) first = FALSE;  
    }
    if (legalmoves <= 1) threat = TRUE;
  }
  /* check for mate / stalemate: */
  if (no_moves) 
  {
    if (in_check ()) 
    {
      StoreTT(-INF+ply, originalalpha, beta, 0, threat, depth);
	  if (DebugRun) printf (" search_ln685 -INF(%d)+ply(%d) return(%d)}\n",-INF,ply,-INF+ply);
      return (-INF+ply);
    }
    else 
    {
      StoreTT(0, originalalpha, beta, 0, threat, depth);
	  if (DebugRun) printf (" search_ln691 return(0)}\n");
      return 0;
    }
  }
  else
  {
     if (fifty > 100) 
     {
	   if (DebugRun) printf (" search_ln685 fifty>100 return(0)}\n");
       return 0;
     }	
  };
  /* doesnt seem to have any effect */ 
  if (sbest == -1) sbest = 255;
  if (best_score <= originalalpha)
  {
    if (!selective) StoreTT(best_score, originalalpha, beta, sbest, threat, depth);
  }
  else 
  { if (!selective) StoreTT(best_score, originalalpha, beta, sbest, threat, depth);
    else StoreTT(best_score, -INF, -INF, sbest, threat, depth);/*store lowbound*/
  }
  if (DebugRun) printf (" search_ln713end best_score return(%d)}\n",best_score);
  return best_score;
}


move_s search_root (int originalalpha, int originalbeta, int depth) 
{
  /* search the root node using alpha-beta with negamax search */
  move_s moves[MOVE_BUFF], best_move;
  int num_moves, i, j;
  long int root_score = -INF, move_ordering[MOVE_BUFF];
  bool no_moves, legal_move, first, found_move;
  int old_ep;
  int alpha, beta;
  move_s kswap;
  int ksswap;
  int incheck;


  best_move = dummy;
  if (DebugRun) printf(" best_move=dummy(%s)",comp_to_coord2(best_move));

  if (DebugRun) printf (" search_root (alpha:%d,beta:%d)\n{\n",originalalpha,originalbeta); 



  alpha = originalalpha;
  beta = originalbeta;

  num_moves = 0;
  no_moves = TRUE;
  ply = 1;
  searching_pv = TRUE;
  time_exit = FALSE;
  time_failure = FALSE;
  first = TRUE;
  cur_score = -INF;
  
  /* check for a draw by 3 fold repetition: */
  if (is_draw ()) 
  {
  	result = draw_by_rep;
	  cur_score = 0;
    pv_length[ply] = 0;

	if (DebugRun) printf ("\n[search_root End draw}");
	return (dummy);
  };
    
  pv_length[ply] = ply;

  /* hash_history[move_number+ply-1] = hash */
  /* check extensions: */ 
  incheck = in_check ();
  checks[ply] = incheck;
  if (incheck) {ext_check++; depth++;};
  /* generate and order moves: */
  gen (&moves[0]); num_moves = numb_moves;
  if (DebugRunST && DebugRunOutput<DebugRunOutputMax) printf (" search_rootL985.order_moves");
  order_moves (&moves[0], &move_ordering[0], num_moves, -1);

  /* loop through the moves at the root: */
  while (remove_one (&i, &move_ordering[0], num_moves)) 
  { make (&moves[0], i);
    if (DebugRun) printf ("\n sr_move[%s]",comp_to_coord2 (moves[i])); 
    legal_move = FALSE; hash_history[move_number+ply-1] = hash;
    path[ply-1] = moves[i]; old_ep = ep_square;
    /* go deeper if it's a legal move: */
    if (check_legal (&moves[0], i)) 
    {   nodes++;
        if ((first == TRUE) || (i_depth < 2))
        {  
          root_score = -search (-beta, -alpha, depth-1, FALSE);
	        ep_square = old_ep;	
	        if (!time_exit && (post || !xb_mode) && i_depth >= mindepth) 
          {   
            if (root_score >= beta)
            { /* update the pv: */
		          pv[ply-1][ply-1] = moves[i];
		          for (j = ply; j < pv_length[ply]; j++) pv[ply-1][j] = pv[ply][j];
		          pv_length[ply-1] = pv_length[ply];
		          post_fh_thinking(root_score, &moves[i]);
            }    
	          else if (root_score <= alpha)
            { 
              //post_fl_thinking(root_score, &moves[i]);                       
            }
	          else
            { /* update the pv: */
		          pv[ply-1][ply-1] = moves[i];
		          for (j = ply; j < pv_length[ply]; j++) pv[ply-1][j] = pv[ply][j];
		          pv_length[ply-1] = pv_length[ply];
		          post_thinking(root_score);
            }
	          if (root_score > cur_score && !time_exit)
            {  cur_score = root_score; bestmovenum = i; 
               best_move = moves[i];
               if (DebugRun) printf(" root_score>cur_score.1.best_move=moves[%d](%s)",i,comp_to_coord2(best_move));
            }  
          }
        }
        else
        { root_score = -search (-alpha-1, -alpha, depth-1, FALSE);
	        ep_square = old_ep;
	        if (root_score > cur_score && !time_exit)
          { cur_score = root_score;  bestmovenum = i;  best_move = moves[i];
            if (DebugRun) printf(" root_score>cur_score.2.best_move=moves[%d](%s)",i,comp_to_coord2(best_move));
          }	
		      if ((root_score > alpha) && (root_score < beta) && !time_exit)
          { 
			      if (i_depth >= mindepth)
            {  /* update the pv: */
		          pv[ply-1][ply-1] = moves[i];
		          for (j = ply; j < pv_length[ply]; j++) pv[ply-1][j] = pv[ply][j];
		          pv_length[ply-1] = pv_length[ply];
		          /*  if (xb_mode) printf("++\n"); */
		          post_fail_thinking(root_score, &moves[i]); 
            } 
	          root_score = -search(-beta, -root_score, depth-1, FALSE);
	          ep_square = old_ep;     
          }
	        if (root_score >= beta && !time_exit) 
	         post_fh_thinking(root_score, &moves[i]);
        }
        if (root_score > cur_score && !time_exit) 
        { cur_score = root_score; bestmovenum = i; best_move = moves[i];
          if (DebugRun) printf(" root_score>cur_score.3.best_move=moves[%d](%s)",i,comp_to_coord2(best_move));
        } 
         // check to see if we've aborted this search before we found a move: 
         // or a failed search <- removed 2000-5-28
         // we should use the fail-highs
         // and the fail-lows are handled in think   
        if (time_exit)
		{ if (no_moves)
	      time_failure = TRUE;
		}  
        no_moves = FALSE;
        legal_move = TRUE;
	}
    unmake (&moves[0], i);
    /* if we've run out of time, return the best we have so far: */
    if (time_exit)
	{	 
       if (DebugRun) printf ("\n[search_root time_exit return best_move %s]\n}\n", comp_to_coord2(best_move));	
	   return best_move;
	};
    /* check our current score vs. alpha: */
    if (root_score > alpha && legal_move) 
    { /* we have a cutoff, so update our killers: */
      /* first, check whether it matches one of the known killers */
      if (moves[i].from == killer1[ply].from && moves[i].target ==
	        killer1[ply].target && moves[i].promoted == killer1[ply].promoted)
	    {  killer_scores[ply]++;
	    }
      else if (moves[i].from == killer2[ply].from && moves[i].target ==
	             killer2[ply].target && moves[i].promoted == killer2[ply].promoted)
	    {   killer_scores2[ply]++;
	        if (killer_scores2[ply] > killer_scores[ply])
	        { kswap = killer1[ply];
	         killer1[ply] = killer2[ply];
	         killer2[ply] = kswap;		
	         ksswap = killer_scores[ply];
	         killer_scores[ply] = killer_scores2[ply];
	         killer_scores2[ply] = ksswap;
	        }
	    }
      else if (moves[i].from == killer3[ply].from && moves[i].target ==
	             killer3[ply].target && moves[i].promoted == killer3[ply].promoted)
	    {   killer_scores3[ply]++;
	        if (killer_scores3[ply] > killer_scores2[ply])
	        {  kswap = killer2[ply];
	          killer2[ply] = killer3[ply];
	          killer3[ply] = kswap;		
	          ksswap = killer_scores2[ply];
	          killer_scores2[ply] = killer_scores3[ply];
	          killer_scores3[ply] = ksswap;
	        }
	    }   
	    /* if not, replace killer3 */
	    else
	    { killer_scores3[ply] = 1; killer3[ply] = moves[i];
	    }
      /* update the history heuristic since we have a cutoff: */
      /* PGC square it */
      history_h[moves[i].from][moves[i].target] += depth * depth;
      alpha = root_score;
      best_move = moves[i];
        if (DebugRun) printf(" alpha=root_score;best_move=moves[%d](%s)",i,comp_to_coord2(best_move));
      bestmovenum = i;
      cur_score = alpha;
      /* update the pv: */
      pv[ply][ply] = moves[i];
      for (j = ply+1; j < pv_length[ply+1]; j++) pv[ply][j] = pv[ply+1][j];
      pv_length[ply] = pv_length[ply+1];
      if (cur_score >= beta) 
	  {	 
		  if (DebugRun) printf ("\n[search_root cur_score(%d)>=beta(%d) return best_move %s]\n}\n", cur_score,beta,comp_to_coord2(best_move));
		  return best_move;
	  };
      /* print out thinking information: */
      if (post && i_depth >= mindepth) 
      { post_thinking (alpha);
      }
    }
    if (legal_move)
      first = FALSE;


    
  }
  /* check to see if we are mated / stalemated: */
  if (no_moves && !is_pondering) 
  {	  if (in_check ()) 
	    {  if (white_to_move == 1) 
		     { result = white_is_mated;
		     }
	       else 
		    { result = black_is_mated;
		    }
	    }
	    else 
	    { result = stalemate;
	    }
  }
  else if (!is_pondering)
  { /* check for draw by the 50 move rule: */
    if (fifty > 100) 
    {  result = draw_by_fifty; cur_score = 0; pv_length[ply] = 0;
       if (DebugRun) printf ("\n[search_root draw_by_fifty return dummy]}\n");
	   return dummy;
    }
  }
  if (DebugRun) printf ("\n[search_root End best_move %s]\n}\n", comp_to_coord2(best_move));

  return best_move;
}


move_s think (void) 
{  /* Perform iterative deepening to go further in the search */ 
  move_s comp_move,comp_moveBook, temp_move;
  int ep_temp, i, j;
  long int elapsed, temp_score, true_score;
  char postmove[STR_BUFF];
  clock_t cpu_start, cpu_end; 
  float et = 0;
  int alpha, beta;
  int tmptmp;
  int rs;
  int failed;
  move_s moves[MOVE_BUFF];
  
  nodes = 0;
  qnodes = 0;
  ply = 0;

  ECacheProbes = 0;
  ECacheHits = 0;
  TTProbes = 0;
  TTHits = 0;
  TTStores = 0;  
  NCuts = 0;
  NTries = 0;
  TExt = 0;
  DeltaTries = 0;
  DeltaCuts = 0;
  FH = 0;
  FHF = 0;
  PVS = 0;
  FULL = 0;
  PVSF = 0;
  ext_check = 0;
  razor_drop = 0;
  razor_material = 0;
  drop_cuts = 0;
  rs = 0;

  true_i_depth = 0;
  bestmovenum = -1;

  //printHolding();

  /* Don't do anything if the queue isn't clean */
  /* PGC: only safe if we're not playing...else partner tells screw us up */
  if (interrupt() && (is_analyzing || is_pondering)) return dummy;
  
  ep_temp = ep_square;
 
  start_time = rtime ();

  gen(&moves[0]);

  ep_square = ep_temp;
  

   /* before we do anything, check to see if we can make a move from book! */
   if (book_ply < 40 && !is_analyzing && !is_pondering) 
   {
     
     comp_moveBook = choose_book_move();
     ep_square = ep_temp;
     /* if choose_book_move() didn't return a junk move indicating that
	            no book move was found, play the book move! :) */
     
     if (comp_moveBook.target == 0) comp_moveBook = choose_binary_book_move();
     ep_square = ep_temp;      
     if (comp_moveBook.target != 0 && !inc ) 
     {
	     comp_to_san (comp_moveBook, postmove);
	     printf("0 0 0 0 %s (Book Move)\n", postmove);
	     cpu_end = clock ();
	     ep_square = ep_temp;
	     time_cushion += (inc*100);
	 
	     return comp_moveBook;
     }
   }
   else 
	  comp_moveBook.target=0;
   
   printf("Material_outgoing:%d\n",Material_outgoing);
   printf("root_to_move:%d\n",root_to_move);

   /* allocate our time for this move: */

   if (!is_pondering)
   {
     if (!fixed_time)
	   {
	     if (go_fast)
       { tmptmp = allocate_time();
	       if (tmptmp > 70)
		     time_for_move = 70;
	       else
		     time_for_move = tmptmp;
       }
	     else
       {  time_for_move = allocate_time ();
	     }	
     }
     else
	   {  time_for_move = fixed_time;
	   }
   }
   else
   {  time_for_move = 999999999;
   };
   if (time_for_move > 100)  LoadLearn();
   pn_move = dummy;
   
   
       /* clear the pv before a new search: */
       for (i = 0; i < PV_BUFF; i++)
	     for (j = 0; j < PV_BUFF; j++)
		   pv[i][j] = dummy;
       
       /* clear the history heuristic: */
       memset (history_h, 0, sizeof (history_h));
       
       /* clear the killer moves: */
       for (i = 0; i < PV_BUFF; i++) 
       {
	       killer_scores[i] = 0;
	       killer_scores2[i] = 0;
	       killer_scores3[i] = 0;
	       killer1[i] = dummy;
	       killer2[i] = dummy;
	       killer3[i] = dummy;
       }
       clear_dp_tt();       
       cpu_start = clock();      
       temp_score = 0;
       cur_score = 0;
       true_score = 0;
       
       for (i_depth = 1; i_depth <= maxdepth; i_depth++) //Brx 4 006p1-p5
       {
	       // don't bother going deeper if we've already used 2/3 of our time, and we
	       //   haven't finished our mindepth search, since we likely won't finsish
	       elapsed = rdifftime (rtime (), start_time);
	       if (elapsed > time_for_move*2.0/3.0 && i_depth > mindepth) break;
	       failed = 0;
	       alpha = temp_score - 100; beta = temp_score + 100;
	       ep_square = ep_temp;
	       temp_move = search_root (alpha, beta, i_depth);
	       if (cur_score <= alpha) failed = 1;
	       if (cur_score <= alpha && !time_exit) /* fail low */
         {  alpha = cur_score -600;
	         beta = cur_score + 1;  rs++;
	         ep_square = ep_temp;
	         temp_move = search_root (alpha, beta, i_depth);
	         if (cur_score > alpha && !time_exit) failed = 0;
	         if (cur_score <= alpha && !time_exit)
           {  alpha = -(INF+1); beta = cur_score + 1; rs++;
		          ep_square = ep_temp;
		          temp_move = search_root (alpha, beta, i_depth);
		          if (cur_score > alpha && !time_exit) failed = 0;
           }
	         else if (cur_score >= beta && !time_exit)
           {  temp_move = search_root (-INF, +INF, i_depth);
              if (!time_exit) failed = 0;
           }
         }
	       else if (cur_score >= beta && !time_exit) /* fail high */
         {  alpha = cur_score - 1; beta = cur_score +600;
	         rs++;
	         ep_square = ep_temp;  temp_move = search_root (alpha, beta, i_depth);
	         if (cur_score >= beta && !time_exit)
           {  alpha = cur_score - 1; beta = +(INF+1); rs++;
              ep_square = ep_temp; temp_move = search_root (alpha, beta, i_depth);
           }
	         else if (cur_score <= alpha && !time_exit)
           {  /* fail high then low...do not make it PV */
		          failed = 1;
           };
         };
	 	     ep_square = ep_temp;
		     if (interrupt() && (i_depth > 1)) 
         { 
	         if (is_pondering)
	           return dummy;
	         else if (!go_fast)
	           break;
         }
	       /* if we haven't aborted our search on time, set the computer's move
	          and post our thinking: */
	       if (!time_failure && !failed) 
         {
	          /* if our search score suddenly drops, and we ran out of time on the
	             search, just use previous results */
	          /* GCP except when we found a mate...maybe generalise ? */
	          /* enabled 2000-5-28 */
	          if (time_exit && (cur_score < temp_score-50) && (cur_score > -900000)) break;
	          /* acidentally pondering if mated */
	          if (cur_score == -INF) return dummy;
		        comp_move = temp_move; temp_score = cur_score; stringize_pv(postpv);
	          if (!time_exit)
            {  true_i_depth = i_depth;
            }      
	          if (i_depth >= mindepth)
            {
	             post_thinking (cur_score);
            }
	          if (temp_score > 900000 && ((int)(1000000-cur_score) < i_depth))
            {   break;
            };
         } //End for
	       if (time_exit) break;
	       /* reset the killer scores (we can keep the moves for move ordering for
	          now, but the scores may not be accurate at higher depths, so we need
	          to reset them): */
	       for (j = 0; j < PV_BUFF; j++) 
         {  killer_scores[j] = 0; killer_scores2[j] = 0; killer_scores3[j] = 0;
         }
	 
       }     
       ep_square = ep_temp;
       if (pn_move.target == dummy.target)
       {
         cpu_end = clock();
         et = (cpu_end-cpu_start)/(double) CLOCKS_PER_SEC;
       };

       /* update our elapsed time_cushion: */
       if (moves_to_tc && !is_pondering) 
       {  elapsed = rdifftime (rtime (), start_time);
          time_cushion += time_for_move-elapsed+inc;
       } 
       if (temp_score == INF-2 && !is_pondering) 
       {
         if (white_to_move == 1) 
         {  result = black_is_mated;
         }
         else 
         {  result = white_is_mated;
         }
       }
       else if (temp_score == -(INF-2) && !is_pondering)
       {
         if (white_to_move == 1)
         { result = white_is_mated;
         }
         else
         {  result = black_is_mated;
         }
       }
       if (post && xb_mode && !is_pondering && 
	         result != black_is_mated &&
	         result != white_is_mated &&
           result != draw_by_fifty && result != draw_by_rep &&
	         result != stalemate && pn_move.target == dummy.target)
       {
          if (temp_score > 900000) 
          {  printf("tellics ptell Mate in %d, give him no more pieces.\n", (int)((1000000-temp_score)/2));
          }   
       }
       if ((temp_score < -999996) && pn_move.target == dummy.target)
       {
          must_sit = TRUE;
          // shut up if the mate is already played
          if (temp_score > -1000000)
          {
	           if (partnerdead)
             {
	              printf("tellics kibitz Both players dead...resigning...\n");
	              printf("tellics resign\n");
             }
	           else
             {
	              printf("tellics ptell I'll have to sit...(dead)\n");
             }
          }
       }
       else if ((temp_score > -60000) && (temp_score < -40000) && !partnerdead && pn_move.target == dummy.target)
       {
          must_sit = TRUE;
          printf("tellics ptell I'll have to sit...(lose piece that mates you)\n");
       }
       else if (inc && comp_moveBook.target) 
       {
	        comp_to_san (comp_moveBook, postmove);
	        printf("0 0 0 0 %s (Book Move)\n", postmove);
	        cpu_end = clock ();
	        ep_square = ep_temp;
	        time_cushion += (inc*100);
	        return comp_moveBook;
       }
       return comp_move;
}
 

void tree (int depth, int indent, FILE *output, char *disp_b) 
{

  move_s moves[MOVE_BUFF];
  int num_moves, i, j, ep_temp;

  ep_temp = ep_square;
  num_moves = 0;

  /* return if we are at the maximum depth: */
  if (!depth) 
  { return;
  }


  /* generate the move list: */
  gen (&moves[0]);
  num_moves = numb_moves;

  /* loop through the moves at the current depth: */
  for (i = 0; i < num_moves; i++) 
  {
    make (&moves[0], i);
    /* check to see if our move is legal: */
    if (check_legal (&moves[0], i)) 
	{
      /* indent and print out our line: */
      for (j = 0; j < indent; j++) 
	  { putc (' ', output);
      }
      print_move (&moves[0], i, output);
      fprintf (output, "\n");

      /* display board if desired: */
      if (disp_b[0] == 'y') display_board (output, 1);

      /* go deeper into the tree recursively, increasing the indent to
	     create the "tree" effect: */
      tree (depth-1, indent+2, output, disp_b);
    }

    /* unmake the move to go onto the next: */
    unmake(&moves[0], i);
  }

  ep_square = ep_temp;

}


